// Modified version by Kotilo. All modifications are under GPLv2. Original:
// Copyright (C) 2002-2005 Nikolaus Gebhardt
// This file is part of the "Irrlicht Engine" and the "irrXML" project.
// For conditions of distribution and use, see copyright notice in irrlicht.h and irrXML.h

#ifndef __IRR_ARRAY_H_INCLUDED__
#define __IRR_ARRAY_H_INCLUDED__

#include "irrTypes.h"
#include "irrString.h"
#include "segments.h"


//!	Self reallocating template array (like stl vector) with additional features.
/** Some features are: Heap sorting, binary search methods, easier debugging.
*/
class char_array
{

public:

	char_array() Seg5;

	//! Constructs a array and allocates an initial chunk of memory.
	//! \param start_count: Amount of elements to allocate.
	char_array(u32 start_count) Seg5;


	//! Copy constructor
	char_array(const char_array& other)  Seg5;	



	//! Destructor. Frees allocated memory, if set_free_when_destroyed
	//! was not set to false by the user before.
	~char_array() Seg5;



	//! Reallocates the array, make it bigger or smaller.
	//! \param new_size: New size of array.
	void reallocate(u32 new_size) Seg5;

	//! Adds an element at back of array. If the array is to small to 
	//! add this new element, the array is made bigger.
	//! \param element: Element to add at the back of the array.
	void push_back(const c8& element) Seg5;


	//! Adds an element at the front of the array. If the array is to small to 
	//! add this new element, the array is made bigger. Please note that this
	//! is slow, because the whole array needs to be copied for this.
	//! \param element: Element to add at the back of the array.
	void push_front(const c8& element) Seg5;

	
	//! Insert item into array at specified position. Please use this
	//! only if you know what you are doing (possible performance loss). 
	//! The preferred method of adding elements should be push_back().
	//! \param element: Element to be inserted
	//! \param index: Where position to insert the new element.
	void insert(const c8& element, u32 index=0)  Seg5;



	//! Clears the array and deletes all allocated memory.
	void clear() Seg5;


	//! Sets pointer to new array, using this as new workspace.
	//! \param newPointer: Pointer to new array of elements.
	//! \param size: Size of the new array.
	void set_pointer(c8* newPointer, u32 size) Seg5;



	//! Sets if the array should delete the memory it used.
	//! \param f: If true, the array frees the allocated memory in its
	//! destructor, otherwise not. The default is true.
	void set_free_when_destroyed(bool f) Seg5;



	//! Sets the size of the array.
	//! \param usedNow: Amount of elements now used.
	void set_used(u32 usedNow) Seg5;



	//! Assignement operator
	void operator=(const char_array& other) Seg5;


	//! Direct access operator
	c8& operator [](u32 index) Seg5;



	//! Direct access operator
	const c8& operator [](u32 index) const Seg5;

    //! Gets last frame
	const c8& getLast() const Seg5;
	
    //! Gets last frame
	c8& getLast() Seg5;
    

	//! Returns a pointer to the array.
	//! \return Pointer to the array.
	c8* pointer() Seg5;



	//! Returns a const pointer to the array.
	//! \return Pointer to the array.
	const c8* const_pointer() const Seg5;


	//! Returns size of used array.
	//! \return Size of elements in the array.
	u32 size() const Seg5;


	//! Returns amount memory allocated.
	//! \return Returns amount of memory allocated. The amount of bytes
	//! allocated would  be allocated_size() * sizeof(ElementsUsed);
	u32 allocated_size() const Seg5;


	//! Returns true if array is empty
	//! \return True if the array is empty, false if not.
	bool empty() const Seg5;



	//! Sorts the array using heapsort. There is no additional memory waste and
	//! the algorithm performs (O) n log n in worst case.
	void sort() Seg5;



	//! Performs a binary search for an element, returns -1 if not found.
	//! The array will be sorted before the binary search if it is not
	//! already sorted.
	//! \param element: Element to search for.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 binary_search(const c8& element) Seg5;


	//! Performs a binary search for an element, returns -1 if not found.
	//! The array will be sorted before the binary search if it is not
	//! already sorted.
	//! \param element: Element to search for.
	//! \param left: First left index
	//! \param right: Last right index.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 binary_search(const c8& element, s32 left, s32 right) Seg5;


	//! Finds an element in linear time, which is very slow. Use
	//! binary_search for faster finding. Only works if =operator is implemented.
	//! \param element: Element to search for.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 linear_search(c8& element) Seg5;


	//! Finds an element in linear time, which is very slow. Use
	//! binary_search for faster finding. Only works if =operator is implemented.
	//! \param element: Element to search for.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 linear_reverse_search(c8& element) Seg5;



	//! Erases an element from the array. May be slow, because all elements 
	//! following after the erased element have to be copied.
	//! \param index: Index of element to be erased.
	void erase(u32 index) Seg5;


	//! Erases some elements from the array. may be slow, because all elements 
	//! following after the erased element have to be copied.
	//! \param index: Index of the first element to be erased.
	//! \param count: Amount of elements to be erased.
	void erase(u32 index, s32 count) Seg5;


	//! Sets if the array is sorted
	void set_sorted(bool _is_sorted) Seg5;
			
	private:

		c8* data;
		u32 allocated;
		u32 used;
		bool free_when_destroyed;
		bool is_sorted;
};



















//!	Self reallocating template array (like stl vector) with additional features.
/** Some features are: Heap sorting, binary search methods, easier debugging.
*/
class estring_array
{

public:

	estring_array() Seg5;

	//! Constructs a array and allocates an initial chunk of memory.
	//! \param start_count: Amount of elements to allocate.
	estring_array(u32 start_count) Seg5;


	//! Copy constructor
	estring_array(const estring_array& other) Seg5;



	//! Destructor. Frees allocated memory, if set_free_when_destroyed
	//! was not set to false by the user before.
	~estring_array() Seg5;


	//! Reallocates the array, make it bigger or smaller.
	//! \param new_size: New size of array.
	void reallocate(u32 new_size) Seg5;

	//! Adds an element at back of array. If the array is to small to 
	//! add this new element, the array is made bigger.
	//! \param element: Element to add at the back of the array.
	void push_back(const estring& element) Seg5;


	//! Adds an element at the front of the array. If the array is to small to 
	//! add this new element, the array is made bigger. Please note that this
	//! is slow, because the whole array needs to be copied for this.
	//! \param element: Element to add at the back of the array.
	void push_front(const estring& element) Seg5;

	
	//! Insert item into array at specified position. Please use this
	//! only if you know what you are doing (possible performance loss). 
	//! The preferred method of adding elements should be push_back().
	//! \param element: Element to be inserted
	//! \param index: Where position to insert the new element.
	void insert(const estring& element, u32 index=0)  Seg5;



	//! Clears the array and deletes all allocated memory.
	void clear() Seg5;



	//! Sets pointer to new array, using this as new workspace.
	//! \param newPointer: Pointer to new array of elements.
	//! \param size: Size of the new array.
	void set_pointer(estring* newPointer, u32 size) Seg5;



	//! Sets if the array should delete the memory it used.
	//! \param f: If true, the array frees the allocated memory in its
	//! destructor, otherwise not. The default is true.
	void set_free_when_destroyed(bool f) Seg5;


	//! Sets the size of the array.
	//! \param usedNow: Amount of elements now used.
	void set_used(u32 usedNow) Seg5;


	//! Assignement operator
	void operator=(const estring_array& other) Seg5;


	//! Direct access operator
	estring& operator [](u32 index) Seg5;



	//! Direct access operator
	const estring& operator [](u32 index) const Seg5;

    //! Gets last frame
	const estring& getLast() const Seg5;

    //! Gets last frame
	estring& getLast() Seg5;
    

	//! Returns a pointer to the array.
	//! \return Pointer to the array.
	estring* pointer() Seg5;


	//! Returns a const pointer to the array.
	//! \return Pointer to the array.
	const estring* const_pointer() const Seg5;



	//! Returns size of used array.
	//! \return Size of elements in the array.
	u32 size() const Seg5;



	//! Returns amount memory allocated.
	//! \return Returns amount of memory allocated. The amount of bytes
	//! allocated would  be allocated_size() * sizeof(ElementsUsed);
	u32 allocated_size() const  Seg5;



	//! Returns true if array is empty
	//! \return True if the array is empty, false if not.
	bool empty() const Seg5;



	//! Sorts the array using heapsort. There is no additional memory waste and
	//! the algorithm performs (O) n log n in worst case.
	void sort() Seg5;



	//! Performs a binary search for an element, returns -1 if not found.
	//! The array will be sorted before the binary search if it is not
	//! already sorted.
	//! \param element: Element to search for.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 binary_search(const estring& element) Seg5;


	//! Performs a binary search for an element, returns -1 if not found.
	//! The array will be sorted before the binary search if it is not
	//! already sorted.
	//! \param element: Element to search for.
	//! \param left: First left index
	//! \param right: Last right index.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 binary_search(const estring& element, s32 left, s32 right) Seg5;


	//! Finds an element in linear time, which is very slow. Use
	//! binary_search for faster finding. Only works if =operator is implemented.
	//! \param element: Element to search for.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 linear_search(estring& element) Seg5;


	//! Finds an element in linear time, which is very slow. Use
	//! binary_search for faster finding. Only works if =operator is implemented.
	//! \param element: Element to search for.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 linear_reverse_search(estring& element) Seg5;



	//! Erases an element from the array. May be slow, because all elements 
	//! following after the erased element have to be copied.
	//! \param index: Index of element to be erased.
	void erase(u32 index) Seg5;


	//! Erases some elements from the array. may be slow, because all elements 
	//! following after the erased element have to be copied.
	//! \param index: Index of the first element to be erased.
	//! \param count: Amount of elements to be erased.
	void erase(u32 index, s32 count) Seg5;

	//! Sets if the array is sorted
	void set_sorted(bool _is_sorted) Seg5;
			
	private:

		estring* data;
		u32 allocated;
		u32 used;
		bool free_when_destroyed;
		bool is_sorted;
};



















//!	Self reallocating template array (like stl vector) with additional features.
/** Some features are: Heap sorting, binary search methods, easier debugging.
*/
class SAttribute_array
{

public:

	SAttribute_array() Seg5;

	//! Constructs a array and allocates an initial chunk of memory.
	//! \param start_count: Amount of elements to allocate.
	SAttribute_array(u32 start_count) Seg5;


	//! Copy constructor
	SAttribute_array(const SAttribute_array& other) Seg5;



	//! Destructor. Frees allocated memory, if set_free_when_destroyed
	//! was not set to false by the user before.
	~SAttribute_array() Seg5;


	//! Reallocates the array, make it bigger or smaller.
	//! \param new_size: New size of array.
	void reallocate(u32 new_size) Seg5;

	//! Adds an element at back of array. If the array is to small to 
	//! add this new element, the array is made bigger.
	//! \param element: Element to add at the back of the array.
	void push_back(const SAttribute& element) Seg5;


	//! Adds an element at the front of the array. If the array is to small to 
	//! add this new element, the array is made bigger. Please note that this
	//! is slow, because the whole array needs to be copied for this.
	//! \param element: Element to add at the back of the array.
	void push_front(const SAttribute& element) Seg5;

	
	//! Insert item into array at specified position. Please use this
	//! only if you know what you are doing (possible performance loss). 
	//! The preferred method of adding elements should be push_back().
	//! \param element: Element to be inserted
	//! \param index: Where position to insert the new element.
	void insert(const SAttribute& element, u32 index=0)  Seg5;



	//! Clears the array and deletes all allocated memory.
	void clear() Seg5;



	//! Sets pointer to new array, using this as new workspace.
	//! \param newPointer: Pointer to new array of elements.
	//! \param size: Size of the new array.
	void set_pointer(SAttribute* newPointer, u32 size) Seg5;



	//! Sets if the array should delete the memory it used.
	//! \param f: If true, the array frees the allocated memory in its
	//! destructor, otherwise not. The default is true.
	void set_free_when_destroyed(bool f) Seg5;


	//! Sets the size of the array.
	//! \param usedNow: Amount of elements now used.
	void set_used(u32 usedNow) Seg5;



	//! Assignement operator
	void operator=(const SAttribute_array& other) Seg5;


	//! Direct access operator
	SAttribute& operator [](u32 index) Seg5;


	//! Direct access operator
	const SAttribute& operator [](u32 index) const Seg5;

    //! Gets last frame
	const SAttribute& getLast() const Seg5;

    //! Gets last frame
	SAttribute& getLast() Seg5;
    

	//! Returns a pointer to the array.
	//! \return Pointer to the array.
	SAttribute* pointer() Seg5;


	//! Returns a const pointer to the array.
	//! \return Pointer to the array.
	const SAttribute* const_pointer() const Seg5;


	//! Returns size of used array.
	//! \return Size of elements in the array.
	u32 size() const Seg5;


	//! Returns amount memory allocated.
	//! \return Returns amount of memory allocated. The amount of bytes
	//! allocated would  be allocated_size() * sizeof(ElementsUsed);
	u32 allocated_size() const Seg5;



	//! Returns true if array is empty
	//! \return True if the array is empty, false if not.
	bool empty() const Seg5;



	//! Sorts the array using heapsort. There is no additional memory waste and
	//! the algorithm performs (O) n log n in worst case.
	void sort() Seg5;



	//! Performs a binary search for an element, returns -1 if not found.
	//! The array will be sorted before the binary search if it is not
	//! already sorted.
	//! \param element: Element to search for.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 binary_search(const SAttribute& element) Seg5;


	//! Performs a binary search for an element, returns -1 if not found.
	//! The array will be sorted before the binary search if it is not
	//! already sorted.
	//! \param element: Element to search for.
	//! \param left: First left index
	//! \param right: Last right index.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 binary_search(const SAttribute& element, s32 left, s32 right) Seg5;


	//! Finds an element in linear time, which is very slow. Use
	//! binary_search for faster finding. Only works if =operator is implemented.
	//! \param element: Element to search for.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 linear_search(SAttribute& element) Seg5;


	//! Finds an element in linear time, which is very slow. Use
	//! binary_search for faster finding. Only works if =operator is implemented.
	//! \param element: Element to search for.
	//! \return Returns position of the searched element if it was found,
	//! otherwise -1 is returned.
	s32 linear_reverse_search(SAttribute& element) Seg5;



	//! Erases an element from the array. May be slow, because all elements 
	//! following after the erased element have to be copied.
	//! \param index: Index of element to be erased.
	void erase(u32 index) Seg5;

	//! Erases some elements from the array. may be slow, because all elements 
	//! following after the erased element have to be copied.
	//! \param index: Index of the first element to be erased.
	//! \param count: Amount of elements to be erased.
	void erase(u32 index, s32 count) Seg5;


	//! Sets if the array is sorted
	void set_sorted(bool _is_sorted) Seg5;
			
	private:

		SAttribute* data;
		u32 allocated;
		u32 used;
		bool free_when_destroyed;
		bool is_sorted;
};





#endif

