// Modified version by Kotilo. All modifications are under GPLv2. Original:
// Copyright (C) 2002-2005 Nikolaus Gebhardt
// This file is part of the "Irrlicht Engine" and the "irrXML" project.
// For conditions of distribution and use, see copyright notice in irrlicht.h and irrXML.h

#ifndef __IRR_STRING_H_INCLUDED__
#define __IRR_STRING_H_INCLUDED__

#include "irrTypes.h"
#include "segments.h"


//!	Very simple string class with some useful features.
/**	string<c8> and string<wchar_t> work both with unicode AND ascii,
so you can assign unicode to string<c8> and ascii to string<wchar_t> 
(and the other way round) if your ever would want to. 
Note that the conversation between both is not done using an encoding.

Known bugs:
Special characters like '', '' and '' are ignored in the
methods make_upper, make_lower and equals_ignore_case.
*/

class estring
{
public:

	//! Default constructor
	estring() Seg4;

	//! Constructor
	estring(const estring& other) Seg4;

	//! Constructs a string from an int
	estring(int number) Seg4;

	//! Constructor for copying a string from a pointer with a given lenght
	estring(const c8* c, s32 lenght) Seg4;

	//! Constructor for unicode and ascii strings
	estring(const c8* c) Seg4;

	//! destructor
	~estring() Seg4;


	//! Assignment operator
	estring& operator=(const estring& other) Seg4;

	//! Assignment operator for strings, ascii and unicode
	estring& operator=(const c8* c) Seg4;

	//! Add operator for other strings
	estring operator+(const estring& other) Seg4;

	//! Add operator for strings, ascii and unicode 
	estring operator+(const c8* c) Seg4;

	//! Direct access operator
	c8& operator [](const s32 index)  const Seg4;

	//! Comparison operator
	bool operator ==(const c8* str) const Seg4;

	//! Comparison operator
	bool operator ==(const estring& other) const Seg4;

	//! Is smaller operator
	bool operator <(const estring& other) const Seg4;


	//! Equals not operator
	bool operator !=(const estring& other) const Seg4;
    
	//! Returns length of string
	/** \return Returns length of the string in characters. */
	s32 size() const Seg4;

	//! Returns character string
	/** \return Returns pointer to C-style zero terminated string. */
	const c8* c_str() const Seg4;

	//! Makes the string lower case.
	void make_lower() Seg4;

	//! Makes the string upper case.
	void make_upper() Seg4;

	//! Compares the string ignoring case.
	/** \param other: Other string to compare.
	\return Returns true if the string are equal ignoring case. */
	bool equals_ignore_case(const estring& other) const Seg4;

	//! compares the first n characters of the strings
	bool equalsn(const estring& other, int len) Seg4;

	//! compares the first n characters of the strings
	bool equalsn(const c8* str, int len) Seg4;

	//! Appends a character to this string
	/** \param character: Character to append. */
	void append(c8 character) Seg4;

	//! Appends a string to this string
	/** \param other: String to append. */
	void append(const estring& other) Seg4;

	//! Appends a string of the length l to this string.
	/** \param other: other String to append to this string.
	 \param length: How much characters of the other string to add to this one. */
	void append(const estring& other, s32 length) Seg4;

	//! Reserves some memory.
	/** \param count: Amount of characters to reserve. */
	void reserve(s32 count) Seg4;

	//! finds first occurrence of character in string
	/** \param c: Character to search for.
	\return Returns position where the character has been found,
	or -1 if not found. */
	s32 findFirst(c8 c) const Seg4;

	//! finds first occurrence of a character of a list in string
	/** \param c: List of strings to find. For example if the method
	should find the first occurance of 'a' or 'b', this parameter should be "ab".
	\param count: Amount of characters in the list. Ususally, 
	this should be strlen(ofParameter1)
	\return Returns position where one of the character has been found,
	or -1 if not found. */
	s32 findFirstChar(c8* c, int count) const Seg4;

	//! Finds first position of a character not in a given list.
	/** \param c: List of characters not to find. For example if the method
	 should find the first occurance of a character not 'a' or 'b', this parameter should be "ab".
	\param count: Amount of characters in the list. Ususally, 
	this should be strlen(ofParameter1)
	\return Returns position where the character has been found,
	or -1 if not found. */
	s32 findFirstCharNotInList(const c8* c, int count) const Seg4;

	//! Finds last position of a character not in a given list.
	/** \param c: List of characters not to find. For example if the method
	 should find the first occurance of a character not 'a' or 'b', this parameter should be "ab".
	\param count: Amount of characters in the list. Ususally, 
	this should be strlen(ofParameter1)
	\return Returns position where the character has been found,
	or -1 if not found. */
	s32 findLastCharNotInList(const c8* c, int count) const Seg4;

	//! finds next occurrence of character in string
	/** \param c: Character to search for.
	\param startPos: Position in string to start searching. 
	\return Returns position where the character has been found,
	or -1 if not found. */
	s32 findNext(c8 c, s32 startPos) const Seg4;	

	//! finds last occurrence of character in string
	//! \param c: Character to search for.
	//! \return Returns position where the character has been found,
	//! or -1 if not found.
	s32 findLast(c8 c) const Seg4;
	


	//! Returns a substring
	//! \param begin: Start of substring.
	//! \param length: Length of substring.
	estring subString(s32 begin, s32 length) Seg4;

	void operator += (c8 c) Seg4;

	void operator += (const estring& other) Seg4;

	void operator += (int i) Seg4;

	//! replaces all characters of a special type with another one
	void replace(c8 toReplace, c8 replaceWith) Seg4;

	//! trims the string.
	/** Removes whitespace from begin and end of the string. */
	void trim() Seg4;


	//! Erases a character from the string. May be slow, because all elements 
	//! following after the erased element have to be copied.
	//! \param index: Index of element to be erased.
	void erase(int index) Seg4;
    	

private:

	//! Returns a character converted to lower case
	c8 toLower(const c8& t) const Seg4;

	//! Reallocate the array, make it bigger or smaler
	void reallocate(s32 new_size) Seg4;

	//--- member variables

	c8* array;
	s32 allocated;
	s32 used;
};

// structure for storing attribute-name pairs
	class SAttribute
	{
        public:
		estring Name;
		estring Value;

        	//! Is smaller operator
    	bool operator <(const SAttribute& other) const Seg4;

        //! Comparison operator
    	bool operator ==(const SAttribute& other) const Seg4;

    	//! Equals not operator
    	bool operator !=(const SAttribute& other) const Seg4;
    
	};





#endif

