import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import javax.imageio.ImageIO;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;


public class QuestionHandler extends DefaultHandler {
	private static final Logger logger   = Logger.getLogger(QuestionHandler.class.getName());
	private BufferedImage bufferedImage  = null;
	private Graphics2D graphics2D        = null;
	private final int PALM_SCREEN_WIDTH  = 160;
	private final int PALM_SCREEN_HEIGHT = 160;
	private String imageFileName;
	private String imageFileFormat       = "png";
	private Color  palmBackgroundColor   = new Color(212, 234, 236);
	private Color  palmForegroundColor   = new Color(0,   0,   0);

	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		super.characters(ch, start, length);
	}

	@Override
	public void endDocument() throws SAXException {
		super.endDocument();			
	}

	@Override
	public void endElement(String uri, String localName, String name) throws SAXException {
		super.endElement(uri, localName, name);

		if ("question".equals(name) && bufferedImage != null) {
			writeImageToDisk();
			bufferedImage = null;
		}
	}

	@Override
	public void startDocument() throws SAXException {
		super.startDocument();
	}

	@Override
	public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {
		super.startElement(uri, localName, name, attributes);

		if ("question".equals(name)) {			
			bufferedImage = new BufferedImage(PALM_SCREEN_WIDTH, PALM_SCREEN_HEIGHT, BufferedImage.TYPE_INT_RGB);
			graphics2D    = (Graphics2D)bufferedImage.getGraphics();
			imageFileName = "question" + attributes.getValue("id") + "." + imageFileFormat;			
			graphics2D.setColor(palmBackgroundColor);
			graphics2D.fillRect(0, 0, PALM_SCREEN_WIDTH, PALM_SCREEN_HEIGHT);
			graphics2D.setColor(palmForegroundColor);
		}

		if ("button".equals(name)) {
			int x       = StringToIntWithDefault(attributes.getValue("x"));
			int y       = StringToIntWithDefault(attributes.getValue("y"));
			int width   = StringToIntWithDefault(attributes.getValue("width"));
			int height  = StringToIntWithDefault(attributes.getValue("height"));
			String text = attributes.getValue("text");
			String font = null;
			if (attributes.getIndex("font") > -1) {
				font = attributes.getValue("text");
			}

			drawButton(x, y, width, height, text, font);
		}

		if ("label".equals(name)) {
			int x       = StringToIntWithDefault(attributes.getValue("x"));
			int y       = StringToIntWithDefault(attributes.getValue("y"));			
			String text = attributes.getValue("text");
			String font = null;
			if (attributes.getIndex("font") > -1) {
				font = attributes.getValue("text");
			}

			drawLabel(x, y, text, font);
		}

		if ("slider".equals(name)) {
			int x     = StringToIntWithDefault(attributes.getValue("x"));
			int y     = StringToIntWithDefault(attributes.getValue("y"));
			int width = StringToIntWithDefault(attributes.getValue("width"));

			int height = 15;
			if (attributes.getIndex("height") > -1) {
				height = StringToIntWithDefault(attributes.getValue("height"));
			}

			int min   = StringToIntWithDefault(attributes.getValue("min"));
			int max   = StringToIntWithDefault(attributes.getValue("max"));
			int step  = StringToIntWithDefault(attributes.getValue("step"));
			int value = StringToIntWithDefault(attributes.getValue("value"));

			drawSlider(x, y, width, height, min, max, step, value);
		}
	}

	void drawButton(int x, int y, int width, int height, String text, String font) {
		graphics2D.drawRoundRect(x, y, width, height, 10, 10);	
		setFont(font);		
		graphics2D.drawString(text, x + 10, y + (height/2) + 5);
	}

	void drawLabel(int x, int y, String text, String font) {
		setFont(font);		
		graphics2D.drawString(text, x, y);
	}

	void drawSlider(int x, int y, int width, int height, int min, int max, int step, int value) {
		// Draw the main part of the slider
		graphics2D.drawRoundRect(x, y, width, height, 5, 5);
		graphics2D.drawLine(x, y+(height/2), x+width, y+(height/2));

		// Draw the ticks
		for (int i = min; i <= max; i += step) {
			if (i == min || i == max) {
				graphics2D.drawLine(x + i, y+height, x + i, y+height+12);
				graphics2D.drawLine(x + i - 2, y+height+12, x + i + 2, y+height+12);			
			} else  if (i % 20 == 0) {
				graphics2D.drawLine(x + i, y+height, x + i, y+height+8);
			}
		}

		// Draw the "handle" of the slider
		graphics2D.fillRect(x+value-5, y-5, 5, height+10);
		graphics2D.fillRect(x+value, y-10, 10, height+20);
		graphics2D.fillRect(x+value+10, y-5, 5, height+10);		
	}

	void setFont(String font) {
		if ("normal".equals(font)) {
			graphics2D.setFont(new Font("Arial", Font.PLAIN, 11));
		} else if ("bold".equals(font)) {
			graphics2D.setFont(new Font("Arial", Font.BOLD, 11));
		} else if ("large".equals(font)) {
			graphics2D.setFont(new Font("Arial", Font.PLAIN, 14));
		} else if ("largeBold".equals(font)) {
			graphics2D.setFont(new Font("Arial", Font.BOLD, 14));
		} else {
			// No font defined. Use normal font as a default.
			graphics2D.setFont(new Font("Arial", Font.PLAIN, 11));
		}
	}

	void writeImageToDisk() {
		try {
			File outputfile = new File(imageFileName);
			ImageIO.write(bufferedImage, imageFileFormat, outputfile);
			System.out.println("Wrote image " + imageFileName);
		} catch (IOException e) {
			logger.warning("Cannot save image: " + imageFileName + " as an " + imageFileFormat + " image.");
		}
	}

	int StringToIntWithDefault(String s) {
		int result = 20;
		try {
			result = Integer.parseInt(s); 
			return result;
		} catch(Exception e) {
			// If string cannot be parsed into integer,
			// return some default value.
			return result;
		}
	}

	//===========================================================
	// SAX ErrorHandler methods
	//===========================================================

	// treat validation errors as fatal
	public void error(SAXParseException e) throws SAXParseException {
		throw e;
	}

	// dump warnings too
	public void warning(SAXParseException err) throws SAXParseException {
		logger.warning("** Warning"
				+ ", line " + err.getLineNumber()
				+ ", uri "  + err.getSystemId()
				+ "   "     + err.getMessage());
	}		
}
