#include "PalmUSB.h"
#include <PalmOS.h>

#define BUFFER_SIZE 256

/***********************************************************************
 *
 *	Global variables
 *
 ***********************************************************************/
UInt16 				gPortId = 0;
SrmOpenConfigType 	gConfig;
Err 				gErr;
char*				gErrorMsg;
char				gBuffer[BUFFER_SIZE];
int					gNumReceived = 0;


void waitMillis(int millis)
{
	int ticksPerSec = SysTicksPerSecond();
	
	if (ticksPerSec)
		SysTaskDelay(millis * ticksPerSec / 1000);
	else
		SysTaskDelay(millis / 10);
}

void close()
{
	if (gPortId) {
		int result = SrmClose(gPortId);
		
		switch (result)
		{
			case errNone:
				gErrorMsg = "Port closed";
				gPortId = 0;				
				return;	
			break;
			
			case serErrBadPort:
				gErrorMsg = "This port doesn't exist";
			break;
			
			case serErrNotOpen:
				gErrorMsg = "The serial port is not open";
			break;
			
			case serErrNoDevicesAvail:
				gErrorMsg = "No serial devices could be found";
			break;
			
			default:
			break;
		}
				
		waitMillis(500);		
		SrmClose(gPortId); /*Retry, it might be hard to die*/		
		gPortId = 0;
	}
}

void connect()
{	
	int err;	
	
	close();
	
	MemSet(&gConfig, sizeof(gConfig), 0);
	gConfig.function = serFncUndefined;

	err = SrmExtOpen(serPortCradleUSBPort , &gConfig, sizeof(gConfig), &gPortId); 
	
	if (err != 0)
	{				
		if (err == serErrBadPort) gErrorMsg = "Connection: BAD PORT";
		if (err == serErrBadParam) gErrorMsg = "Connection: BAD PARAM";
		if (err == serErrAlreadyOpen) gErrorMsg = "Connection: ALREADY OPEN";
		if (err == memErrNotEnoughSpace) gErrorMsg = "Connection: NOT ENOUGH SPACE";
	}
}

void sent(char* message)
{
	UInt32 toSend, numSent; 
	
	toSend = StrLen(message); 
	
	if (gPortId) {		
		numSent = SrmSend(gPortId, message, toSend, &gErr); 
		if (gErr == serErrTimeOut) { 
			gErrorMsg = "Send: ERROR TIMEOUT";
		}
		if (numSent != toSend) {
			gErrorMsg = "Send: ONLY PARTIAL";
		}
	}
}

void receive(unsigned long timeout)
{
    MemSet(&gBuffer, sizeof(gBuffer), '\0');
    gNumReceived = 0;
	
	if (gPortId) {
		gNumReceived = SrmReceive(gPortId, gBuffer, BUFFER_SIZE, timeout, &gErr); 
	
			switch (gErr) {
				case errNone:
					gErrorMsg = "No error";
				break;
				
				case serErrBadPort :
					gErrorMsg = "This port doesn't exist";
				break;
				
				case serErrNotOpen :
					gErrorMsg = "The port is not open";
				break;
				
				case serErrTimeOut :
					gErrorMsg = "Unable to receive data within the specified timeout period";
				break;
				
				case serErrConfigurationFailed :
					gErrorMsg = "The port needs time to configure, and the configuration has failed";
				break;
				
				case serErrNotSupported :
					gErrorMsg = "The port is not the foreground port";
				break;
				
				case serErrLineErr :
					gErrorMsg = "A line error occurred during the receipt of data. Use SrmGetStatus() to obtain the exact line error";
				break;
				
				case serErrNoDevicesAvail :
					gErrorMsg = "No serial devices could be found";
				break;
				
				default:
				break;
			}			
	}
}

void usb_send(char* message)
{
	connect();
	
    if (gPortId) {
   		SrmSendFlush(gPortId); 	
   		SrmReceiveFlush(gPortId, SysTicksPerSecond() * 1.5);
    }
   
	sent(message); 
	
	if (gPortId) {
		waitMillis(1000);
	}
	
	close();	
}

void usb_receive(unsigned long timeout)
{
	connect();   		
	receive(timeout); 	
	close();
}

void usb_clear_errors()
{
	gErrorMsg = 0;
}

char* usb_get_error_message()
{
	return gErrorMsg;
}

char* usb_get_message()
{
	return gBuffer;
}

int   usb_get_message_length()
{
	return gNumReceived;
}

void  usb_open()
{
	connect();
}

void  usb_close()
{
	close();
}

void  usb_bulk_send(char* message)
{
    if (gPortId) {
   		SrmSendFlush(gPortId); 	
   		SrmReceiveFlush(gPortId, SysTicksPerSecond() * 1.5);
    }
   
	sent(message); 
	
	if (gPortId) {
		waitMillis(1000);
	}
	
}

void  usb_bulk_receive(unsigned long timeout)
{
	receive(timeout); 	
}
